#[derive(Clone, Copy)]
pub(crate) struct EmbeddedAsset {
    pub(crate) path: &'static str,
    pub(crate) content_type: &'static str,
    pub(crate) bytes: &'static [u8],
}

const FALLBACK_INDEX_HTML: &[u8] = br#"<!doctype html>
<html lang=\"en\">
  <head><meta charset=\"UTF-8\"/><meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0\"/><title>llmfit</title></head>
  <body style=\"font-family: sans-serif; padding: 24px\">
    <h1>llmfit Web Dashboard</h1>
    <p>Frontend assets are missing.</p>
    <p>From repo root run: <code>cd llmfit-web && npm ci && npm run build</code></p>
    <script src=\"/assets/fallback.js\"></script>
  </body>
</html>"#;
const FALLBACK_JS: &[u8] = br"console.warn('llmfit-web dist assets not found; serving fallback page.');";

pub(crate) static EMBEDDED_WEB_ASSETS: &[EmbeddedAsset] = &[
    EmbeddedAsset { path: "/index.html", content_type: "text/html; charset=utf-8", bytes: FALLBACK_INDEX_HTML },
    EmbeddedAsset { path: "/assets/fallback.js", content_type: "text/javascript; charset=utf-8", bytes: FALLBACK_JS },
];
